@@ -1,13 +1,16 @@
 # RDF::Trine::Error
+# -------------
+# $Revision: 127 $
+# $Date: 2006-02-08 14:53:21 -0500 (Wed, 08 Feb 2006) $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
 
-RDF::Trine::Error - Error classes for RDF::Trine
+RDF::Trine::Error - Error classes for RDF::Trine.
 
 =head1 VERSION
 
-This document describes RDF::Trine::Error version 0.999_02
+This document describes RDF::Trine::Error version 1.001
 
 =head1 SYNOPSIS
 
@@ -15,9 +18,8 @@ This document describes RDF::Trine::Error version 0.999_02
 
 =head1 DESCRIPTION
 
-RDF::Trine::Error provides a class hierarchy of errors that other RDF::Trine
-classes may throw using the L<Error|Error> API. See L<Error> for more
-information.
+RDF::Trine::Error provides an class hierarchy of errors that other RDF::Trine
+classes may throw using the L<Error|Error> API. See L<Error> for more information.
 
 =head1 REQUIRES
 
@@ -36,9 +38,11 @@ use base qw(Error);
 
 ######################################################################
 
-our ($VERSION);
+our ($REVISION, $VERSION, $debug);
 BEGIN {
-	$VERSION	= '0.999_02';
+	$debug		= 0;
+	$REVISION	= do { my $REV = (qw$Revision: 127 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
+	$VERSION	= 1.001;
 }
 
 ######################################################################
@@ -59,35 +63,13 @@ package RDF::Trine::Error::SerializationError;
 
 use base qw(RDF::Trine::Error);
 
-package RDF::Trine::Error::DatabaseError;
-
-use base qw(RDF::Trine::Error);
-
-package RDF::Trine::Error::ParserError;
-
-use base qw(RDF::Trine::Error);
-
-package RDF::Trine::Error::UnimplementedError;
-
-use base qw(RDF::Trine::Error);
 
 1;
 
 __END__
 
-=head1 BUGS
-
-Please report any bugs or feature requests to through the GitHub web interface
-at L<https://github.com/kasei/perlrdf/issues>.
-
 =head1 AUTHOR
 
-Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
-
-=head1 COPYRIGHT
-
-Copyright (c) 2006-2012 Gregory Todd Williams. This
-program is free software; you can redistribute it and/or modify it under
-the same terms as Perl itself.
+ Gregory Williams <gwilliams@cpan.org>
 
 =cut
