@@ -1,13 +1,17 @@
 # RDF::Trine::Iterator
+# -------------
+# $Revision $
+# $Date $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
 
-RDF::Trine::Iterator - Iterator class for SPARQL query results
+RDF::Trine::Iterator - Stream (iterator) class for SPARQL query results.
 
 =head1 VERSION
 
-This document describes RDF::Trine::Iterator version 0.999_02.
+This document describes RDF::Trine::Iterator version 1.000.
+
 
 =head1 SYNOPSIS
 
@@ -30,19 +34,21 @@ use strict;
 use warnings;
 no warnings 'redefine';
 
+use JSON;
+use XML::Twig;
 use Data::Dumper;
-use Log::Log4perl;
 use Carp qw(carp);
-use Scalar::Util qw(blessed reftype refaddr);
+use List::MoreUtils qw(uniq);
+use Scalar::Util qw(blessed reftype);
 
-use XML::SAX;
 use RDF::Trine::Node;
-use RDF::Trine::Iterator::SAXHandler;
-use RDF::Trine::Iterator::JSONHandler;
 
-our ($VERSION, @ISA, @EXPORT_OK);
+our ($REVISION, $VERSION, $debug, @ISA, @EXPORT_OK);
+use constant DEBUG	=> 0;
 BEGIN {
-	$VERSION	= '0.999_02';
+	$debug		= DEBUG;
+	$REVISION	= do { my $REV = (qw$Revision: 293 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
+	$VERSION	= '1.000';
 	
 	require Exporter;
 	@ISA		= qw(Exporter);
@@ -81,7 +87,7 @@ sub new {
 	my $names		= shift || [];
 	my %args		= @_;
 	
-	if (ref($stream) and ref($stream) eq 'ARRAY') {
+	if (ref($stream) and reftype($stream) eq 'ARRAY') {
 		my $array	= $stream;
 		$stream	= sub {
 			return shift(@$array);
@@ -99,9 +105,7 @@ sub new {
 		_names		=> $names,
 		_stream		=> $stream,
 		_args		=> \%args,
-		_count		=> 0,
 		_row		=> undef,
-		_peek		=> [],
 #		_source		=> Carp::longmess(),
 	};
 	
@@ -152,7 +156,7 @@ Other options are:
 
 sub to_string {
 	my $self	= shift;
-	my $format	= shift || 'http://www.w3.org/2005/sparql-results#';
+	my $format	= shift || 'http://www.w3.org/2001/sw/DataAccess/rf1/result2';
 	if (ref($format) and $format->isa('RDF::Redland::URI')) {
 		$format	= $format->as_string;
 	}
@@ -164,7 +168,7 @@ sub to_string {
 	}
 }
 
-=item C<< from_string ( $xml ) >>
+=item C<from_string ( $xml )>
 
 Returns a new iterator using the supplied XML in the SPARQL XML Results format.
 
@@ -173,73 +177,69 @@ Returns a new iterator using the supplied XML in the SPARQL XML Results format.
 sub from_string {
 	my $class	= shift;
 	my $string	= shift;
-	unless (ref($string)) {
-		my $data	= $string;
-		open( my $fh, '<', \$data );
-		$string	= $fh;
+	
+	my $bool;
+	my @vars;
+	my $value;
+	my @results;
+	my $result	= {};
+	
+	my $twig	= XML::Twig->new(
+		twig_handlers => {
+			variable	=> sub { push(@vars, $_->att('name')) },
+			binding		=> sub { $result->{ $_->att('name') }	= $value; },
+			result		=> sub { push(@results, $result); $result	= {}; },
+			bnode		=> sub { $value	= RDF::Trine::Node::Blank->new( $_->text ) },
+			uri			=> sub { $value	= RDF::Trine::Node::Resource->new( $_->text ) },
+			literal		=> sub {
+							my $lang	= $_->att('xml:lang');
+							my $dt		= $_->att('datatype');
+							$value	= RDF::Trine::Node::Literal->new( $_->text, $lang, $dt )
+						},
+			boolean		=> sub { $bool	= ($_->text eq 'true') ? 1 : 0 },
+		},
+	);
+	$twig->parse( $string );
+	
+	if (defined($bool)) {
+		return RDF::Trine::Iterator::Boolean->new( [$bool] );
+	} else {
+		return RDF::Trine::Iterator::Bindings->new( \@results, \@vars );
 	}
-	my $handler	= RDF::Trine::Iterator::SAXHandler->new();
-	my $p		= XML::SAX::ParserFactory->parser(Handler => $handler);
-	$p->parse_file( $string );
-	my $iter	= $handler->iterator;
-	return $iter;
 }
 
-=item C<< from_json ( $json ) >>
-
-=cut
-
-sub from_json {
-	my $class	= shift;
-	my $json	= shift;
-	my $p		= RDF::Trine::Iterator::JSONHandler->new( @_ );
-	return $p->parse( $json );
-}
-
-
-=item C<< next_result >>
-
 =item C<< next >>
 
+=item C<< next_result >>
+
 Returns the next item in the stream.
 
 =cut
 
-sub next_result { $_[0]->next }
-sub next {
+sub next { $_[0]->next_result }
+sub next_result {
 	my $self	= shift;
 	return if ($self->{_finished});
 	
-	if (scalar(@{ $self->{_peek} })) {
-		return shift(@{ $self->{_peek} });
-	}
-	
 	my $stream	= $self->{_stream};
 	my $value	= $stream->();
 	unless (defined($value)) {
 		$self->{_finished}	= 1;
 	}
 
+	my $args	= $self->_args;
+# 	if ($args->{named}) {
+# 		if ($self->_bridge->supports('named_graph')) {
+# 			my $bridge	= $self->_bridge;
+# 			$args->{context}	= $bridge->get_context( $self->{_stream}, %$args );
+# 		}
+# 	}
+	
 	$self->{_open}	= 1;
 	$self->{_row}	= $value;
-	$self->{_count}++ if defined($value);
 	return $value;
 }
 
-=item C<< peek >>
-
-Returns the next value from the iterator without consuming it. The value will
-remain in queue until the next call to C<< next >>.
-
-=cut
-
-sub peek {
-	my $self	= shift;
-	return if ($self->{_finished});
-	my $value	= $self->next;
-	push( @{ $self->{_peek} }, $value );
-	return $value;
-}
 
 =item C<< current >>
 
@@ -267,8 +267,6 @@ Returns true if the end of the stream has been reached, false otherwise.
 sub end { $_[0]->finished }
 sub finished {
 	my $self	= shift;
-	my $v		= $self->peek;
-	return 0 if (defined($v));
 	return $self->{_finished};
 }
 
@@ -297,6 +295,22 @@ sub close {
 	return;
 }
 
+# =item C<< context >>
+# 
+# Returns the context node of the current result (if applicable).
+# 
+# =cut
+# 
+# sub context {
+# 	my $self	= shift;
+# 	my $args	= $self->_args;
+# 	my $bridge	= $args->{bridge};
+# 	my $stream	= $self->{_stream};
+# 	my $context	= $bridge->get_context( $stream, %$args );
+# 	return $context;
+# }
+
+
 =item C<< concat ( $stream ) >>
 
 Returns a new stream resulting from the concatenation of the referant and the
@@ -326,17 +340,6 @@ sub concat {
 	return $s;
 }
 
-=item C<< seen_count >>
-
-Returns the count of elements that have been returned by this iterator at the
-point of invocation.
-
-=cut
-
-sub seen_count {
-	my $self	= shift;
-	return $self->{_count};
-}
 
 =item C<get_boolean>
 
@@ -346,7 +349,7 @@ Returns the boolean value of the first item in the stream.
 
 sub get_boolean {
 	my $self	= shift;
-	my $data	= $self->next;
+	my $data	= $self->next_result;
 	return +$data;
 }
 
@@ -366,6 +369,71 @@ sub get_all {
 	return @data;
 }
 
+=item C<< join_streams ( $stream, $stream ) >>
+
+Performs a natural, nested loop join of the two streams, returning a new stream
+of joined results.
+
+=cut
+
+sub join_streams {
+	my $self	= shift;
+	my $astream	= shift;
+	my $bstream	= shift;
+#	my $bridge	= shift;
+	my %args	= @_;
+	
+#	my $debug	= $args{debug};
+	
+	Carp::confess unless ($astream->isa('RDF::Trine::Iterator::Bindings'));
+	Carp::confess unless ($bstream->isa('RDF::Trine::Iterator::Bindings'));
+	
+	my @names	= uniq( map { $_->binding_names() } ($astream, $bstream) );
+	my $a		= $astream->project( @names );
+	my $b		= $bstream->project( @names );
+	
+	my @results;
+	my @data	= $b->get_all();
+	no warnings 'uninitialized';
+	while (my $rowa = $a->next) {
+		LOOP: foreach my $rowb (@data) {
+			warn "[--JOIN--] " . join(' ', map { my $row = $_; '{' . join(', ', map { join('=',$_,$row->{$_}->as_string) } (keys %$row)) . '}' } ($rowa, $rowb)) . "\n" if ($debug);
+			my %keysa	= map {$_=>1} (keys %$rowa);
+			my @shared	= grep { $keysa{ $_ } } (keys %$rowb);
+			foreach my $key (@shared) {
+				my $val_a	= $rowa->{ $key };
+				my $val_b	= $rowb->{ $key };
+				my $defined	= 0;
+				foreach my $n ($val_a, $val_b) {
+					$defined++ if (defined($n));
+				}
+				if ($defined == 2) {
+					unless ($val_a->equal($val_b)) {
+						warn "can't join because mismatch of $key (" . join(' <==> ', map {$_->as_string} ($val_a, $val_b)) . ")" if ($debug);
+						next LOOP;
+					}
+				}
+			}
+			
+			my $row	= { (map { $_ => $rowa->{$_} } grep { defined($rowa->{$_}) } keys %$rowa), (map { $_ => $rowb->{$_} } grep { defined($rowb->{$_}) } keys %$rowb) };
+			if ($debug) {
+				warn "JOINED:\n";
+				foreach my $key (keys %$row) {
+					warn "$key\t=> " . $row->{ $key }->as_string . "\n";
+				}
+			}
+			push(@results, $row);
+		}
+	}
+	
+	my $args	= $astream->_args;
+	return $astream->_new( \@results, 'bindings', \@names, %$args );
+}
+
+
+
+
+
 =begin private
 
 =item C<format_node_xml ( $node, $name )>
@@ -386,28 +454,20 @@ sub format_node_xml ($$$$) {
 	my $node_label;
 	
 	if(!defined $node) {
-		return '';
+		$node_label	= "<unbound/>";
 	} elsif ($node->is_resource) {
 		$node_label	= $node->uri_value;
 		$node_label	=~ s/&/&amp;/g;
 		$node_label	=~ s/</&lt;/g;
 		$node_label	=~ s/"/&quot;/g;
 		$node_label	= qq(<uri>${node_label}</uri>);
-	} elsif ($node->isa('RDF::Trine::Node::Literal')) {
+	} elsif ($node->is_literal) {
 		$node_label	= $node->literal_value;
 		$node_label	=~ s/&/&amp;/g;
 		$node_label	=~ s/</&lt;/g;
 		$node_label	=~ s/"/&quot;/g;
-		if ($node->has_language) {
-			my $lang	= $node->literal_value_language;
-			$node_label	= qq(<literal xml:lang="${lang}">${node_label}</literal>);
-		} elsif ($node->has_datatype) {
-			my $dt	= $node->literal_datatype;
-			$node_label	= qq(<literal datatype="${dt}">${node_label}</literal>);
-		} else {
-			$node_label	= qq(<literal>${node_label}</literal>);
-		}
-	} elsif ($node->isa('RDF::Trine::Node::Blank')) {
+		$node_label	= qq(<literal>${node_label}</literal>);
+	} elsif ($node->is_blank) {
 		$node_label	= $node->blank_identifier;
 		$node_label	=~ s/&/&amp;/g;
 		$node_label	=~ s/</&lt;/g;
@@ -431,23 +491,7 @@ sub construct_args {
 	my $self	= shift;
 	my $type	= $self->type;
 	my $args	= $self->_args || {};
-	return ($type, [], %$args);
-}
-
-=item C<< each ( \&callback ) >>
-
-Calls the callback function once for each item in the iterator, passing the
-item as an argument to the function. Any arguments to C<< each >> beyond the
-callback function will be passed as supplemental arguments to the callback
-function.
-
-=cut
-
-sub each {
-	my ($self, $coderef) = (shift, shift);
-	while (my $row = $self->next) {
-		$coderef->($row, @_);
-	}
+	return ($type, []);
 }
 
 =begin private
@@ -476,6 +520,25 @@ sub _row {
 	return $self->{_row};
 }
 
+# =item C<< bridge >>
+# 
+# Returns the model bridge object used for insepcting the objects returned by the stream.
+# 
+# =cut
+# 
+# sub bridge {
+# 	my $self	= shift;
+# 	if (@_) {
+# 		$self->_args->{bridge}	= shift;
+# 	}
+# 	return $self->_args->{bridge};
+# }
+# 
+# sub _bridge {
+# 	my $self	= shift;
+# 	return $self->bridge;
+# }
+
 sub _names {
 	my $self	= shift;
 	return $self->{_names};
@@ -487,6 +550,10 @@ sub _stream {
 }
 
 
+
+
+
+
 =back
 
 =head1 FUNCTIONS
@@ -543,13 +610,6 @@ sub smap (&$;$$$) {
 	my $block	= shift;
 	my $stream	= shift;
 	my @args	= $stream->construct_args();
-	foreach my $i (0 .. $#args) {
-		last unless (scalar(@_));
-		my $new	= shift;
-		if (defined($new)) {
-			$args[ $i ]	= $new;
-		}
-	}
 	my $class	= ref($stream);
 	
 	my $open	= 1;
@@ -617,21 +677,17 @@ L<JSON|JSON>
 
 L<Scalar::Util|Scalar::Util>
 
-L<XML::SAX|XML::SAX>
-
-=head1 BUGS
-
-Please report any bugs or feature requests to through the GitHub web interface
-at L<https://github.com/kasei/perlrdf/issues>.
 
 =head1 AUTHOR
 
-Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
+Gregory Todd Williams  C<< <greg@evilfunhouse.com> >>
 
-=head1 COPYRIGHT
 
-Copyright (c) 2006-2012 Gregory Todd Williams. This
-program is free software; you can redistribute it and/or modify it under
-the same terms as Perl itself.
+=head1 LICENCE AND COPYRIGHT
+
+Copyright (c) 2007, Gregory Todd Williams C<< <gwilliams@cpan.org> >>. All rights reserved.
+
+This module is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself. See L<perlartistic>.
+
 
-=cut
