@@ -1,19 +1,15 @@
 # RDF::Trine::Model::StatementFilter
+# -------------
+# $Revision: 121 $
+# $Date: 2006-02-06 23:07:43 -0500 (Mon, 06 Feb 2006) $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
 
 RDF::Trine::Model::StatementFilter - Model for filtering statements based on a user-specified criteria
 
-=head1 VERSION
-
-This document describes RDF::Trine::Model::StatementFilter version 0.999_02
-
 =head1 METHODS
 
-Beyond the methods documented below, this class inherits methods from the
-L<RDF::Trine::Model> class.
-
 =over 4
 
 =cut
@@ -29,11 +25,12 @@ use Scalar::Util qw(blessed reftype);
 
 use RDF::Trine::Node;
 use RDF::Trine::Pattern;
+use RDF::Trine::Store::DBI;
 use RDF::Trine::Iterator qw(sgrep);
 
-our ($VERSION);
+our $debug;
 BEGIN {
-	$VERSION	= '0.999_02';
+	$debug	= 0;
 }
 
 ################################################################################
@@ -123,17 +120,16 @@ sub get_pattern {
 		
 		my @streams;
 		foreach my $triple (@triples) {
-			my @vars	= map { $_->name } grep { $_->isa('RDF::Trine::Node::Variable') } $triple->nodes;
 			Carp::confess "not a statement object: " . Dumper($triple) unless ($triple->isa('RDF::Trine::Statement'));
 			my $stream	= $self->get_statements( $triple->nodes, $context );
-			my $binds	= $stream->as_bindings( $triple->nodes )->project( @vars );
+			my $binds	= $stream->as_bindings( $triple->nodes );
 			push(@streams, $binds);
 		}
 		if (@streams) {
 			while (@streams > 1) {
 				my $a	= shift(@streams);
 				my $b	= shift(@streams);
-				unshift(@streams, RDF::Trine::Iterator::Bindings->join_streams( $a, $b ));
+				unshift(@streams, RDF::Trine::Iterator->join_streams( $a, $b ));
 			}
 		} else {
 			push(@streams, RDF::Trine::Iterator::Bindings->new([{}], []));
@@ -191,19 +187,8 @@ __END__
 
 =back
 
-=head1 BUGS
-
-Please report any bugs or feature requests to through the GitHub web interface
-at L<https://github.com/kasei/perlrdf/issues>.
-
 =head1 AUTHOR
 
-Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
-
-=head1 COPYRIGHT
-
-Copyright (c) 2006-2012 Gregory Todd Williams. This
-program is free software; you can redistribute it and/or modify it under
-the same terms as Perl itself.
+ Gregory Todd Williams <gwilliams@cpan.org>
 
 =cut
