@@ -294,7 +294,7 @@ sub add_statement {
 	my $context	= shift;
 	
 	my $dbh		= $self->dbh;
-	Carp::confess unless (blessed($stmt));
+# 	Carp::confess unless (blessed($stmt));
 	my $stable	= $self->statements_table;
 	my @nodes	= $stmt->nodes;
 	foreach my $n (@nodes) {
@@ -383,7 +383,7 @@ sub _add_node {
 	my @cols;
 	my $table;
 	my %values;
-	Carp::confess unless (blessed($node));
+# 	Carp::confess unless (blessed($node));
 	if ($node->is_blank) {
 		$table	= "Bnodes";
 		@cols	= qw(ID Name);
@@ -701,7 +701,7 @@ sub _sql_for_expr {
 	### None of these should affect the context directly, since the expression
 	### might be a child of another expression (e.g. "isliteral(?node) || isresource(?node)")
 	
-	if ($expr->isa('RDF::Query::Algebra::Expr::Function')) {
+	if ($expr->isa('RDF::Query::Expression::Function')) {
 		my $func	= $expr->uri->uri_value;
 		my @args	= $expr->arguments;
 		if ($func eq 'sparql:isliteral' and blessed($args[0]) and $args[0]->isa('RDF::Trine::Node::Variable')) {
@@ -835,7 +835,8 @@ using the same algorithm that Redland's mysql storage backend uses.
 
 =cut
 
-sub _mysql_hash {
+sub _mysql_hash;
+sub _mysql_hash_pp {
 	my $data	= shift;
 	my @data	= unpack('C*', md5( $data ));
 	my $sum		= Math::BigInt->new('0');
@@ -852,6 +853,14 @@ sub _mysql_hash {
 	return $sum;
 }
 
+BEGIN {
+	eval "use RDF::Trine::XS;";
+	no strict 'refs';
+	*{ '_mysql_hash' }	= (RDF::Trine::XS->can('hash'))
+		? \&RDF::Trine::XS::hash
+		: \&_mysql_hash_pp;
+}
+
 =item C<< _mysql_node_hash ( $node ) >>
 
 Returns a hash value (computed by C<_mysql_hash> for the supplied C<$node>.
@@ -884,8 +893,8 @@ sub _mysql_node_hash {
 	} else {
 		return undef;
 	}
-	
-	my $hash	= _mysql_hash( $data );
+	my $hash;
+	$hash	= _mysql_hash( $data );
 	return $hash;
 }
 
@@ -934,7 +943,7 @@ Returns the name of the underlying model.
 
 sub model_name {
 	my $self	= shift;
-	Carp::confess unless (blessed($self));
+# 	Carp::confess unless (blessed($self));
 	return $self->{model_name};
 }
 
